%
%  TSV_V_ALL.m
%  Copyright (C) 2018 Frederick Sander, Heiner Zille, Sanaz Mostaghim
%
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%  Authors of this Code: 
%   Frederick Sander <frederick.sander@posteo.de>
%   Heiner Zille <heiner.zille@ovgu.de>
%   Sanaz Mostaghim <sanaz.mostaghim@ovgu.de> 
%
%  This class belongs to the following publication:
%
%  Frederick Sander, Heiner Zille and Sanaz Mostaghim 
%  "Transfer strategies from single- to multi-objective grouping mechanisms"
%  Genetic and Evolutionary Computation Conference (GECCO), ACM, Kyoto, Japan, July 2018
%  https://dl.acm.org/citation.cfm?id=3205455.3205491
%
function DVSet = TSV_V_ALL(combMat, DV)
    % start with the first dimension, check all complete groups and
    % if the variable cannot be added to any group, create a new one and
    % assign the variable to it.
    ISet             = {};
    remainingIndices = 1:numel(DV);
    
    %when all indices are added to the DVSet, algorithm is finished
    while ~isempty(remainingIndices)
        %get next index to check
        actIndex            = remainingIndices(1);
        %delete the actual one from the remaining ones
        remainingIndices(1) = [];
        %when no group could be found this flag remians false. False means
        %that a new group will be created which contains only the actIndex.
        %True means that a group was found and actIndex will be added to
        %that group.
        groupFound          = false;
        
        %check every group if the actIndex may fit in. select the first one
        %where all elements interact with actIndex.
        for d = 1:length(ISet)
            %hier ansetzten, DV wird reingegeben, vielleich groupset
            %konvertieren
            actGroup       = ISet{d};
            
            %flag to store if the actGroup TOGETHER with the actIndex form
            %a fully connected graph.
            fullyConnected = true;
            
            %the actIndex is checked against every element of the actGroup.
            %If there exists an element el with no connection to actIndex,
            %the flag fullyConnected is set to false, otherwise it remins
            %true.
            for el = actGroup
                if ~combMat(actIndex, el)
                    fullyConnected = false;
                end
            end
            
            % actIndex interacts with every variable from the actGroup, it
            % is added to the actGroup
            if fullyConnected
                ISet{d} = [ISet{d}, actIndex];
                groupFound = true;
                break;
            end
        end
        
        %if no group was found, create a new group which only contains
        %actindex
        if ~groupFound
            ISet = [ISet, actIndex];
        end
    end
    
    %create the array with the REAl (DV) variables
    DVSet = {};
    for actGroup = ISet  
        DVSet{end+1} = DV(actGroup{1});
    end
end