%
%  TSV_V_SINGLE.m
%  Copyright (C) 2018 Frederick Sander, Heiner Zille, Sanaz Mostaghim
%
%  This program is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%  Authors of this Code: 
%   Frederick Sander <frederick.sander@posteo.de>
%   Heiner Zille <heiner.zille@ovgu.de>
%   Sanaz Mostaghim <sanaz.mostaghim@ovgu.de> 
%
%  This class belongs to the following publication:
%
%  Frederick Sander, Heiner Zille and Sanaz Mostaghim 
%  "Transfer strategies from single- to multi-objective grouping mechanisms"
%  Genetic and Evolutionary Computation Conference (GECCO), ACM, Kyoto, Japan, July 2018
%  https://dl.acm.org/citation.cfm?id=3205455.3205491
%

function DVSet = TSV_V_SINGLE(combMat, DV)
    DVSet            = {};
    %remaining indices which have to be checked
    remainingIndices = 1:numel(DV);
    
    %when all indices are added to the DVSet, algorithm is finished
    while ~isempty(remainingIndices)
        %get the next index to check
        actIndices = remainingIndices(1);
        
        %the actual group consists only of the next index to check
        actGroup   = remainingIndices(1);
     
        %add all connected indices to the actual group and to the array
        %actIndices. All inidices of the actIndices array are also checked
        %if they have connections to any other index.
        while ~isempty(actIndices)
            %the current Index to check
            currentIndex  = actIndices(1);
            %delete the curent Index from the array of the actIndices
            actIndices(1) = [];

            %get all the indices which are connected to the current one
            connected = getConnectedIndicesInRow(combMat, currentIndex);
            
            % for every connected index, add it to the actGroup and actIndices
            % array if it does not appear in any of them.
            for con = connected
                if ~ismember(con, actGroup) && ~ismember(con, actIndices)
                    actGroup   = [actGroup, con];
                    actIndices = [actIndices, con];
                end
            end
        end
        
        %delete all indices of the found group from the remianingIndices
        for el = actGroup
            remainingIndices = remainingIndices(remainingIndices ~= el);
        end
        
        % add the REAL indices to the DVSet. To get the real ones
        % DV(actGroup) is used.
        DVSet{end+1} = DV(actGroup);
    end
end